@@ -124,6 +124,16 @@
 /*	Available in Postfix 3.3 and later:
 /* .IP "\fBservice_name (read-only)\fR"
 /*	The master.cf service name of a Postfix daemon process.
+/* .PP
+/*	Available in Postfix 3.6 and later:
+/* .IP "\fBenable_threaded_bounces (no)\fR"
+/*	Enable non-delivery, success, and delay notifications that link
+/*	to the original message by including a References: and In-Reply-To:
+/*	header with the original Message-ID value.
+/* .PP
+/*	Available in Postfix 3.7 and later:
+/* .IP "\fBheader_from_format (standard)\fR"
+/*	The format of the Postfix-generated \fBFrom:\fR header.
 /* FILES
 /*	/var/spool/postfix/bounce/* non-delivery records
 /*	/var/spool/postfix/defer/* non-delivery records
@@ -150,6 +160,9 @@
 /*	Google, Inc.
 /*	111 8th Avenue
 /*	New York, NY 10011, USA
+/*
+/*	Wietse Venema
+/*	porcupine.org
 /*--*/
 
 /* System library. */
@@ -177,6 +190,7 @@
 #include <mail_addr.h>
 #include <rcpt_buf.h>
 #include <dsb_scan.h>
+#include <hfrom_format.h>
 
 /* Single-threaded server skeleton. */
 
@@ -197,6 +211,8 @@ char   *var_bounce_rcpt;
 char   *var_2bounce_rcpt;
 char   *var_delay_rcpt;
 char   *var_bounce_tmpl;
+bool    var_threaded_bounce;
+char   *var_hfrom_format;		/* header_from_format */
 
  /*
   * We're single threaded, so we can avoid some memory allocation overhead.
@@ -215,6 +231,11 @@ static DSN_BUF *dsn_buf;
   */
 BOUNCE_TEMPLATES *bounce_templates;
 
+ /*
+  * From: header format.
+  */
+int     bounce_hfrom_format;
+
 #define STR vstring_str
 
 #define VS_NEUTER(s) printable(vstring_str(s), '?')
@@ -295,7 +316,7 @@ static int bounce_notify_proto(char *service_name, VSTREAM *client,
 {
     const char *myname = "bounce_notify_proto";
     int     flags;
-    int     smtputf8;
+    int     sendopts;
     int     dsn_ret;
 
     /*
@@ -306,7 +327,7 @@ static int bounce_notify_proto(char *service_name, VSTREAM *client,
 			    RECV_ATTR_STR(MAIL_ATTR_QUEUE, queue_name),
 			    RECV_ATTR_STR(MAIL_ATTR_QUEUEID, queue_id),
 			    RECV_ATTR_STR(MAIL_ATTR_ENCODING, encoding),
-			    RECV_ATTR_INT(MAIL_ATTR_SMTPUTF8, &smtputf8),
+			    RECV_ATTR_INT(MAIL_ATTR_SENDOPTS, &sendopts),
 			    RECV_ATTR_STR(MAIL_ATTR_SENDER, sender),
 			    RECV_ATTR_STR(MAIL_ATTR_DSN_ENVID, dsn_envid),
 			    RECV_ATTR_INT(MAIL_ATTR_DSN_RET, &dsn_ret),
@@ -330,9 +351,9 @@ static int bounce_notify_proto(char *service_name, VSTREAM *client,
     VS_NEUTER(sender);
     VS_NEUTER(dsn_envid);
     if (msg_verbose)
-	msg_info("%s: flags=0x%x service=%s queue=%s id=%s encoding=%s smtputf8=%d sender=%s envid=%s ret=0x%x",
+	msg_info("%s: flags=0x%x service=%s queue=%s id=%s encoding=%s sendopts=%d sender=%s envid=%s ret=0x%x",
 		 myname, flags, service_name, STR(queue_name), STR(queue_id),
-		 STR(encoding), smtputf8, STR(sender), STR(dsn_envid),
+		 STR(encoding), sendopts, STR(sender), STR(dsn_envid),
 		 dsn_ret);
 
     /*
@@ -346,7 +367,7 @@ static int bounce_notify_proto(char *service_name, VSTREAM *client,
      * Execute the request.
      */
     return (service(flags, service_name, STR(queue_name),
-		    STR(queue_id), STR(encoding), smtputf8,
+		    STR(queue_id), STR(encoding), sendopts,
 		    STR(sender), STR(dsn_envid), dsn_ret,
 		    bounce_templates));
 }
@@ -357,7 +378,7 @@ static int bounce_verp_proto(char *service_name, VSTREAM *client)
 {
     const char *myname = "bounce_verp_proto";
     int     flags;
-    int     smtputf8;
+    int     sendopts;
     int     dsn_ret;
 
     /*
@@ -368,7 +389,7 @@ static int bounce_verp_proto(char *service_name, VSTREAM *client)
 			    RECV_ATTR_STR(MAIL_ATTR_QUEUE, queue_name),
 			    RECV_ATTR_STR(MAIL_ATTR_QUEUEID, queue_id),
 			    RECV_ATTR_STR(MAIL_ATTR_ENCODING, encoding),
-			    RECV_ATTR_INT(MAIL_ATTR_SMTPUTF8, &smtputf8),
+			    RECV_ATTR_INT(MAIL_ATTR_SENDOPTS, &sendopts),
 			    RECV_ATTR_STR(MAIL_ATTR_SENDER, sender),
 			    RECV_ATTR_STR(MAIL_ATTR_DSN_ENVID, dsn_envid),
 			    RECV_ATTR_INT(MAIL_ATTR_DSN_RET, &dsn_ret),
@@ -398,9 +419,9 @@ static int bounce_verp_proto(char *service_name, VSTREAM *client)
 	return (-1);
     }
     if (msg_verbose)
-	msg_info("%s: flags=0x%x service=%s queue=%s id=%s encoding=%s smtputf8=%d sender=%s envid=%s ret=0x%x delim=%s",
+	msg_info("%s: flags=0x%x service=%s queue=%s id=%s encoding=%s sendopts=%d sender=%s envid=%s ret=0x%x delim=%s",
 		 myname, flags, service_name, STR(queue_name),
-		 STR(queue_id), STR(encoding), smtputf8, STR(sender),
+		 STR(queue_id), STR(encoding), sendopts, STR(sender),
 		 STR(dsn_envid), dsn_ret, STR(verp_delims));
 
     /*
@@ -418,12 +439,12 @@ static int bounce_verp_proto(char *service_name, VSTREAM *client)
 					  mail_addr_double_bounce())) {
 	msg_warn("request to send VERP-style notification of bounced mail");
 	return (bounce_notify_service(flags, service_name, STR(queue_name),
-				      STR(queue_id), STR(encoding), smtputf8,
+				      STR(queue_id), STR(encoding), sendopts,
 				      STR(sender), STR(dsn_envid), dsn_ret,
 				      bounce_templates));
     } else
 	return (bounce_notify_verp(flags, service_name, STR(queue_name),
-				   STR(queue_id), STR(encoding), smtputf8,
+				   STR(queue_id), STR(encoding), sendopts,
 				   STR(sender), STR(dsn_envid), dsn_ret,
 				   STR(verp_delims), bounce_templates));
 }
@@ -434,7 +455,7 @@ static int bounce_one_proto(char *service_name, VSTREAM *client)
 {
     const char *myname = "bounce_one_proto";
     int     flags;
-    int     smtputf8;
+    int     sendopts;
     int     dsn_ret;
 
     /*
@@ -445,7 +466,7 @@ static int bounce_one_proto(char *service_name, VSTREAM *client)
 			    RECV_ATTR_STR(MAIL_ATTR_QUEUE, queue_name),
 			    RECV_ATTR_STR(MAIL_ATTR_QUEUEID, queue_id),
 			    RECV_ATTR_STR(MAIL_ATTR_ENCODING, encoding),
-			    RECV_ATTR_INT(MAIL_ATTR_SMTPUTF8, &smtputf8),
+			    RECV_ATTR_INT(MAIL_ATTR_SENDOPTS, &sendopts),
 			    RECV_ATTR_STR(MAIL_ATTR_SENDER, sender),
 			    RECV_ATTR_STR(MAIL_ATTR_DSN_ENVID, dsn_envid),
 			    RECV_ATTR_INT(MAIL_ATTR_DSN_RET, &dsn_ret),
@@ -494,9 +515,9 @@ static int bounce_one_proto(char *service_name, VSTREAM *client)
      * RECIPIENT_FROM_RCPT_BUF().
      */
     if (msg_verbose)
-	msg_info("%s: flags=0x%x queue=%s id=%s encoding=%s smtputf8=%d sender=%s envid=%s dsn_ret=0x%x orig_to=%s to=%s off=%ld dsn_orig=%s notif=0x%x stat=%s act=%s why=%s",
+	msg_info("%s: flags=0x%x queue=%s id=%s encoding=%s sendopts=%d sender=%s envid=%s dsn_ret=0x%x orig_to=%s to=%s off=%ld dsn_orig=%s notif=0x%x stat=%s act=%s why=%s",
 		 myname, flags, STR(queue_name), STR(queue_id),
-		 STR(encoding), smtputf8, STR(sender), STR(dsn_envid),
+		 STR(encoding), sendopts, STR(sender), STR(dsn_envid),
 		 dsn_ret, STR(rcpt_buf->orig_addr), STR(rcpt_buf->address),
 		 rcpt_buf->offset, STR(rcpt_buf->dsn_orcpt),
 		 rcpt_buf->dsn_notify, STR(dsn_buf->status),
@@ -506,7 +527,7 @@ static int bounce_one_proto(char *service_name, VSTREAM *client)
      * Execute the request.
      */
     return (bounce_one_service(flags, STR(queue_name), STR(queue_id),
-			       STR(encoding), smtputf8, STR(sender),
+			       STR(encoding), sendopts, STR(sender),
 			       STR(dsn_envid), dsn_ret, rcpt_buf,
 			       dsn_buf, bounce_templates));
 }
@@ -528,11 +549,9 @@ static void bounce_service(VSTREAM *client, char *service_name, char **argv)
 	msg_fatal("malformed service name: %s", service_name);
 
     /*
-     * Announce the protocol. Allow lazy clients to receive this together
-     * with the server's response to the client request, but also allow
-     * greedy clients to detect a service mismatch early.
+     * Announce the protocol.
      */
-    attr_print(client, ATTR_FLAG_MORE,
+    attr_print(client, ATTR_FLAG_NONE,
 	       SEND_ATTR_STR(MAIL_ATTR_PROTO, MAIL_ATTR_PROTO_BOUNCE),
 	       ATTR_TYPE_END);
     (void) vstream_fflush(client);
@@ -615,6 +634,7 @@ static void pre_jail_init(char *unused_name, char **unused_argv)
 
 static void post_jail_init(char *service_name, char **unused_argv)
 {
+    bounce_hfrom_format = hfrom_format_parse(VAR_HFROM_FORMAT, var_hfrom_format);
 
     /*
      * Special case: dump bounce templates. This is not part of the master(5)
@@ -668,6 +688,11 @@ int     main(int argc, char **argv)
 	VAR_2BOUNCE_RCPT, DEF_2BOUNCE_RCPT, &var_2bounce_rcpt, 1, 0,
 	VAR_DELAY_RCPT, DEF_DELAY_RCPT, &var_delay_rcpt, 1, 0,
 	VAR_BOUNCE_TMPL, DEF_BOUNCE_TMPL, &var_bounce_tmpl, 0, 0,
+	VAR_HFROM_FORMAT, DEF_HFROM_FORMAT, &var_hfrom_format, 1, 0,
+	0,
+    };
+    static const CONFIG_NBOOL_TABLE nbool_table[] = {
+	VAR_THREADED_BOUNCE, DEF_THREADED_BOUNCE, &var_threaded_bounce,
 	0,
     };
 
@@ -683,6 +708,7 @@ int     main(int argc, char **argv)
 		       CA_MAIL_SERVER_INT_TABLE(int_table),
 		       CA_MAIL_SERVER_STR_TABLE(str_table),
 		       CA_MAIL_SERVER_TIME_TABLE(time_table),
+		       CA_MAIL_SERVER_NBOOL_TABLE(nbool_table),
 		       CA_MAIL_SERVER_PRE_INIT(pre_jail_init),
 		       CA_MAIL_SERVER_POST_INIT(post_jail_init),
 		       CA_MAIL_SERVER_UNLIMITED,
