@@ -355,7 +355,8 @@ static BOUNCE_INFO *bounce_mail_alloc(const char *service,
 		quote_822_local_flags(bounce_info->sender,
 				      VSTRING_LEN(bounce_info->buf) ?
 				      STR(bounce_info->buf) :
-				      mail_addr_mail_daemon(), 0);
+				      mail_addr_mail_daemon(),
+				      QUOTE_FLAG_8BITCLEAN);
 	    }
 
 	    /*
@@ -532,6 +533,14 @@ int     bounce_header(VSTREAM *bounce, BOUNCE_INFO *bounce_info,
 	post_mail_fprintf(bounce, "In-Reply-To: %s", STR(bounce_info->orig_msgid));
     }
 
+    /*
+     * Trade confidentiality against availability.
+     */
+#if 0
+    if (var_tls_required_enable)
+	post_mail_fprintf(bounce, "TLS-Required: no");
+#endif
+
     /*
      * Auto-Submitted header, as per RFC 3834.
      */
