@@ -518,7 +518,8 @@ static int smtp_get_effective_tls_level(DSN_BUF *why, SMTP_STATE *state)
      * If the sender requires TLS with server certificate verification, the
      * TLS level must enforce a server certificate match.
      */
-    else if ((state->request->sendopts & SOPT_REQUIRETLS_ESMTP)) {
+    else if (var_requiretls_enable
+	     && (state->request->sendopts & SOPT_REQUIRETLS_ESMTP)) {
 	if (TLS_MUST_MATCH(tls->level) == 0) {
 	    dsb_simple(why, "5.7.10", "Sender requires a TLS server "
 		       "certificate match, but the configured %s TLS "
